﻿using System;
using System.Windows.Forms;
using System.IO.Ports;


// I used com0com to set up two virtual comms ports on my computer: CNCA0 and CNCB0 - They appear as COM3 and COM4 in this 
// application's combo box, and in the comms port combo box on the analyser. 
//
// Your port names may vary, depending on how many hardware comms ports you already have on your computer.
//
// 1 - Start this code running
//
// 2 - Select one of the COM ports created up with com0com - ie: COM3
//
// 3 - Select the paired COM in the analyser comms combo box - In this example it would be COM4
// 
// If all goes well, the analyser should respond as if it has found Trackgen running on an Arduino
//
// If you select a comms port on the analyser, before selecting a comms port in this application, you will have to
// click the "Send to analyser" button. This sends "Trackgen V1.0" to the analyser. The analyser sees "Trackgen" and
// initialises the Trackgen system.
// 
// Using the various control options on the Tracking generator panel will cause the analyser to send Trackgen
// commands to this application.
//
// Note: This code is NOT bullet-proof and will allow you to set incorrect coms ports without complaining.
//
// If you set a conflicting comms port in the analyser, for example COM3 in this application, and then COM3 on the analyser,
// the analyser will crash with an exception error - this will be addressed in the next release.
//
// You can download a copy of com0com from here: https://sourceforge.net/projects/com0com/?source=typ_redirect
//
// Have fun !
// 
// Steve Andrew - June 2018



namespace VirtualTrackgen
{
	public partial class frmVirtualTrackgen : Form
	{
		private SerialPort Serial;

		private String analyserMsgStr;			// response from the analyser
		private bool commsSet;					// valid comms port flag
		private bool analyserConnected = false;	// connection established flag

		// need this - comms port runs in it's own thread so it cannot directly manipulate controls in the main thread
		private delegate void postCommsMessage(string message);

		public frmVirtualTrackgen()
		{
			InitializeComponent();

			CommsSetup();
		}

		public void CommsSetup()
		{
			// build a list of the available ports on the computer, including virtual ports

			string[] Ports = SerialPort.GetPortNames();
			foreach (string portName in Ports)
			{
				// ignore com0com names like CNCA0 and CNB0
				// we'll get an exception error if we try to open one of those, so we won't list them
				if (portName.Contains("COM"))
					cmbCommsPorts.Items.Add(portName);
			}
		}

		public void cmbCommsPorts_SelectedIndexChanged(object sender, EventArgs e)
		{
			// select a comms port from the combo box

			bool validPort = true;
			string commName = cmbCommsPorts.Text;

			// if we had a previously selected port, get rid of it and plug the comms handler into the new port
			if (Serial != null)
			{
				Serial.DataReceived -= new SerialDataReceivedEventHandler(AsciiCommsHandler);
				Serial.Dispose();
			}

			// start again with a new port
			Serial = new SerialPort();

			Serial.BaudRate = 115200;
			Serial.ReceivedBytesThreshold = 1;
			Serial.DataReceived += new SerialDataReceivedEventHandler(AsciiCommsHandler);
			Serial.PortName = commName;
			Serial.ReadBufferSize = 16384;
			Serial.ReadTimeout = 500;
			Serial.WriteTimeout = 500;
			 
			analyserConnected = false;

			try
			{
				Serial.Open();
			}
			catch
			{
				validPort = false;
				commsSet = false;
			}

			// if the port opened without an error, then send a message to the analyser.
			// it's waiting for a string containing "Trackgen" - the version number is currently ignored
			if (validPort)
			{
				commsSet = true;

				Serial.DiscardInBuffer();
				Serial.DiscardOutBuffer();
				txtAnalyserMessage.Text = "";

				SendToAnalyser("Trackgen V1.0");
			}
		}

		public void AsciiCommsHandler(object sender, SerialDataReceivedEventArgs e)
		{
			while (Serial.BytesToRead > 0)
			{
				analyserMsgStr = Serial.ReadLine();

				// sometimes happens - @#$%&*! Microsoft comms driver... stream buffer does not always update comms buffer
				// Microsoft confirms this in the serial port documentation
				if (analyserMsgStr.Length == 0)
					return;

				// strip any control chars from the end of the string
				analyserMsgStr = analyserMsgStr.Trim();

				processAnalyserCommand(analyserMsgStr);
			}
		}

		public void processAnalyserCommand(string msg)
		{
			// display whatever the analyser sends to us
			Invoke(new postCommsMessage(updateMessageBox), new object[] { msg });

			// the analyser sends "VERSION" every time a comms port is selected, and waits to receive a string
			// containing "Trackgen"

			if (msg.Contains("VERSION"))
			{
				SendToAnalyser("Trackgen V1.0");	
				analyserConnected = true;
			}

			// the analyser sends us various commands - GETPINS is one of them...
			// The pin data we send will be saved in the analyser's CurrentSettings.ini file 
			// Currently there is no ACK/NAK handshaking implimented - this will be done in the next release.

			// if you try and send more than 4 fields, the analyser will crash - so don't do it !
			// error checking will be addressed in the next release

			if(msg.Contains("GETPINS"))
				SendToAnalyser("R:23  D:25  C:31  U:27");

			// the analyser only excepts two resonses, one is "Trackgen", the other is the above response to GETPINS
			// anything else sent to the analyser will be ignored.
		}

		private void updateMessageBox(string msg)
		{
			txtAnalyserMessage.Text += "\r\n" +  msg;
		}

		private void SendToAnalyser(string data)
		{
			// only do this if we have a valid comms port selected
			if (commsSet)
			{
				try
				{
					Serial.WriteLine(data);
				}
				catch
				{
				}
			}
		}

		private void btnSendToAnalyser_Click(object sender, EventArgs e)
		{
			SendToAnalyser(txtMsgToAnalyser.Text);
		}

		private void btnClear_Click(object sender, EventArgs e)
		{
			txtAnalyserMessage.Text = "";
		}
	}
}
